# Generated by Django 5.0.10 on 2025-07-21 13:23

import dcim.fields
import django.db.models.deletion
import taggit.managers
import utilities.json
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('dcim', '0191_module_bay_rebuild'),
        ('extras', '0121_customfield_related_object_filter'),
        ('ipam', '0070_vlangroup_vlan_id_ranges'),
    ]

    operations = [
        migrations.CreateModel(
            name='MAConPorts',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True, null=True)),
                ('last_updated', models.DateTimeField(auto_now=True, null=True)),
                ('custom_field_data', models.JSONField(blank=True, default=dict, encoder=utilities.json.CustomFieldJSONEncoder)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('mac', dcim.fields.MACAddressField(blank=True)),
                ('status', models.CharField(default='unknown', max_length=50)),
                ('lldp_device', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('netbox_device', models.CharField(blank=True, default=None, max_length=50, null=True)),
                ('device', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dcim.device')),
                ('interface', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dcim.interface')),
                ('ipaddress', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ipam.ipaddress')),
                ('tags', taggit.managers.TaggableManager(through='extras.TaggedItem', to='extras.Tag')),
                ('vlan', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='ipam.vlan')),
            ],
            options={
                'verbose_name_plural': 'MAC and IP on switches ports',
            },
        ),
    ]
